﻿package script{

	/*
	 * AINE - Accelerated Interactive Narrator Engine
	 * @version 0.4.2 ALPHA
	 * @author Daniel Svejstrup Christensen
	 * @see http://www.drakkashi.com/aine/ for updates
	 * This software is released under the MIT License <http://www.opensource.org/licenses/mit-license.php>
	 * Copyright © 2014 Drakkashi.com
	 */

	import flash.display.MovieClip;
	import flash.display.Bitmap;
    import flash.display.Shape;
    import flash.utils.Timer;
	import flash.utils.getQualifiedClassName;
	import flash.utils.getDefinitionByName;
	import flash.events.MouseEvent;
	import flash.events.Event;
    import flash.events.TimerEvent;
	import flash.text.TextFieldAutoSize;
	import fl.transitions.Tween;
	import fl.transitions.easing.*;

	public class Interface extends MovieClip{
		
		private static var ui:Interface,
						   navSystem:String = "compass";

		private var navButtons:Array,
					gameOver:GameOver,
					gameMenu:GameMenu = new GameMenu(),
					playerImage:Bitmap,
					playerItems:Array = new Array(),
					roomImage:Bitmap,
					roomItems:Array = new Array(),
					charList:Array = new Array(),
					tooltip:MovieClip,
					tooltipStr:String,
					tooltipTimer:Timer = new Timer(1000,0),
					eventList:MovieClip,
					eventListRect:Shape,
					eventListObj:_Object,
					eventListText:Array = new Array();


		public function Interface(){
			if (navSystem == "compass"){
				navCross.visible = false;
				btn_in.x = 219,05;
				navButtons = new Array(navCompass.btn_north,navCompass.btn_northEast,navCompass.btn_east,navCompass.btn_southEast,
											  navCompass.btn_south,navCompass.btn_southWest,navCompass.btn_west,navCompass.btn_northWest,
											  navCompass.btn_up,navCompass.btn_down,btn_in,btn_out
					);
			}
			else{
				navCompass.visible = false;
				navButtons = new Array(navCross.btn_forward,null,navCross.btn_right,null,navCross.btn_back,null,
											  navCross.btn_left,null,navCross.btn_up,navCross.btn_down,btn_in,btn_out
					);
			}
			
			for (var i:int = 0; i < navButtons.length; i++){
				if (navButtons[i]){
					navButtons[i].addEventListener(MouseEvent.CLICK,eventNavClick);				
					navButtons[i].addEventListener(MouseEvent.ROLL_OVER,eventNavOver);				
					navButtons[i].addEventListener(MouseEvent.ROLL_OUT,eventNavOut);				
				}
			}

			roomImageHolder.addEventListener(MouseEvent.CLICK,eventMouseClick);
			roomImageHolder.addEventListener(MouseEvent.ROLL_OVER,eventMouseOver);
			roomImageHolder.addEventListener(MouseEvent.ROLL_OUT,eventMouseOut);
			player.imageHolder.addEventListener(MouseEvent.CLICK,eventMouseClick);
			player.imageHolder.addEventListener(MouseEvent.ROLL_OVER,eventMouseOver);
			player.imageHolder.addEventListener(MouseEvent.ROLL_OUT,eventMouseOut);
			tooltipTimer.addEventListener(TimerEvent.TIMER, showTooltip);
			addEventListener(MouseEvent.CLICK,hideTooltip);

			updatePlayerPortrait();
			updatePlayerItems();
			updateRoom();
			ui = this;
			Engine.getStage().addChild(this);
		}

		public function updateRoom():void{
			updateRoomImage();
			updateRoomItems();
			updateRoomName();
			updateCharList();
			updateNav();
		}

		private function eventNavClick(e:MouseEvent):void{
			var room:Room = getNavRoom(e.currentTarget);
			if (e.currentTarget.currentFrame == 1 && !room.triggerEvent("NAV_CLICK")){
				room.enterRoom();
			}
		}

		private function eventNavOver(e:MouseEvent):void{
			var room:Room = getNavRoom(e.currentTarget);
			if (e.currentTarget.currentFrame == 1 && !room.triggerEvent("NAV_OVER")){
				setTooltipString(room.getTooltip());
				tooltipTimer.start();
			}
		}

		private function eventNavOut(e:MouseEvent):void{
			var room:Room = getNavRoom(e.currentTarget);
			if (e.currentTarget.currentFrame == 1 && !room.triggerEvent("NAV_OUT"))
				hideTooltip();
		}

		private function getNavRoom(obj:Object):Room{
			return Room.getCurrent().getNav().getRoom(navButtons.indexOf(obj));
		}


		private function eventMouseClick(e:MouseEvent):void{
			var obj:_Object = getObject(e.currentTarget);
			if (!obj.triggerEvent("MOUSE_CLICK"))
				showEvents(obj);
		}

		private function eventMouseOver(e:MouseEvent):void{
			var obj:_Object = getObject(e.currentTarget);
			if (!obj.triggerEvent("MOUSE_OVER") && e.currentTarget != roomImageHolder && obj != eventListObj){
				setTooltipString(obj.getTooltip());
				tooltipTimer.start();
			}
		}

		private function eventMouseOut(e:MouseEvent):void{
			var obj:_Object = getObject(e.currentTarget);
			if (!obj.triggerEvent("MOUSE_OUT"))
				hideTooltip();
		}

		private function getObject(obj:Object):_Object{
			if (obj == roomImageHolder)
				return Room.getCurrent();
			if (obj == player.imageHolder)
				return Player.getPlayer();
			if (charList.indexOf(obj) >= 0)
				return charList[charList.indexOf(obj)];
			if (playerItems.indexOf(obj) >= 0)
				return playerItems[playerItems.indexOf(obj)];
			if (roomItems.indexOf(obj) >= 0 )
				return roomItems[roomItems.indexOf(obj)];
			return null;
		}

		public function updatePlayerPortrait():void{
			var image:Bitmap = Image.getImage(Player.getPlayer().getImage());
			
			if (playerImage)
				player.imageHolder.removeChild(playerImage);

			if (image){
				if (image.width > image.height){
					image.height = image.height/image.width*119.1;
					image.width = 119.1;
				}
				else{
					image.width = image.width/image.height*119.1;
					image.height = 119.1;
				}

				image.x = 6.7 + (119.1-image.width)/2;
				image.y = 6.7 + (119.1-image.height)/2;
				player.imageHolder.addChild(image);
				playerImage = image;
			}
			else{
				var gender:String = Player.getPlayer().getGender();

				if (gender == "female"){
					/*
					 * DEFAULT FEMALE IMAGE
					 */
				}
				else if (gender == "male"){
					/*
					 * DEFAULT MALE IMAGE
					 */
				}
			}
		}

		public function updateRoomName():void{
			txt_title.text = Room.getCurrent().getName();
		}

		public function updateRoomImage():void{
			var image:Bitmap = Image.getImage(Room.getCurrent().getImage());

			if (image){
				if (roomImage && roomImage.parent)
					roomImageHolder.removeChild(roomImage);

				if (image.width / 800 > image.height / 495){
					image.width = image.width/image.height*495;
					image.height = 495;
				}
				else{
					image.height = image.height/image.width*800;
					image.width = 800;
				}

				image.x = (800-image.width)/2;
				image.y = (495-image.height)/2;
				image = Image.cropImage(image);
				roomImageHolder.addChild(image);
				roomImage = image;
			}
			else if (roomImage){
				if (roomImage.parent)
					roomImageHolder.removeChild(roomImage);
				roomImage = null;
			}
		}

		public function showEvents(obj:_Object):void{
			if (obj != eventListObj){
				if (eventList)
					hideEvents();

				var customList:Array = obj.getCustomList();

				if (customList.length > 0){
					eventList = new MovieClip;
					eventListRect = new Shape;
					eventListObj = obj;

					var frame:Shape = new Shape,
						rect:Shape = new Shape;
	
					eventList.addChild(frame);
					eventList.addChild(rect);
					eventList.addChild(eventListRect);

					eventListText = new Array();
					for (var i:int = 0; i < customList.length; i++){
						var txt:TextField = new TextField();
						txt.text = customList[i].getEvent();
						txt.autoSize = TextFieldAutoSize.LEFT;
						txt.mouseEnabled = false;
						txt.x = 25;
						txt.y = txt.height*(eventListText.length);
						eventListText.push(txt);
						eventList.addChild(txt);
					}

					rect.graphics.beginFill(0xFFFFFF,1);
					rect.graphics.drawRect(0, 0, eventList.width+50,eventListText[0].height*eventListText.length);
					rect.graphics.endFill();
					frame.graphics.beginFill(0xFFFFFF,0);
					frame.graphics.lineStyle(0.1,0x262626,1);
					frame.graphics.drawRect(0, 0, rect.width,rect.height);
					frame.graphics.endFill();
					eventListRect.graphics.beginFill(0xDFDFDF,1);
					eventListRect.graphics.drawRect(0, 0, rect.width,eventListText[0].height);
					eventListRect.graphics.endFill();
					eventListRect.visible = false;
					eventList.addChild(frame);

					eventList.x = (mouseX + eventList.width <= 800 ? mouseX : mouseX-1 -eventList.width);
					eventList.y = (mouseY + eventList.height <= 480 ? mouseY : mouseY - eventList.height);

					addEventListener(MouseEvent.CLICK,eventListClick);
					eventList.addEventListener(MouseEvent.ROLL_OVER,eventListOver);
					eventList.addEventListener(MouseEvent.ROLL_OUT,eventListOut);
					eventList.addEventListener(MouseEvent.MOUSE_MOVE,eventListMove);
					new Tween(eventList,"alpha",None.easeNone,0,1,1,false);
					addChild(eventList);
				}
			}
			else
				hideEvents();
		}

		private function hideEvents():void{
			if (eventList && eventList.alpha > 0){
				removeEventListener(MouseEvent.CLICK,eventListClick);
				eventList.removeEventListener(MouseEvent.ROLL_OVER,eventListOver);
				eventList.removeEventListener(MouseEvent.ROLL_OUT,eventListOut);
				eventList.removeEventListener(MouseEvent.ROLL_OUT,eventListMove);

				if (eventList.parent)
					removeChild(eventList);
				eventList = null;
				eventListObj = null;
			}
		}

		private function eventListClick(e:MouseEvent):void{
			if (mouseX > eventList.x && mouseX < eventList.x + eventList.width &&
				mouseY > eventList.y && mouseY < eventList.y + eventList.height
			)
				eventListObj.getCustomList()[getIndex()].trigger();
			hideEvents();
		}

		private function eventListOver(e:MouseEvent):void{
			eventListMove();
			eventListRect.visible = true;
		}

		private function eventListOut(e:MouseEvent):void{
			eventListRect.visible = false;
		}

		private function eventListMove(e:MouseEvent = null):void{
			eventListRect.y = getIndex()*eventListText[0].height;
		}

		private function getIndex():int{
			return (mouseY - eventList.y-1)/eventListText[0].height;
		}

		private function setTooltipString(str:String):void{
			str = trim(str);
			if (str.length > 50 && str.indexOf(".") >= 0 && str.substring(0,str.indexOf(".")).length < 50)
				tooltipStr = str.substring(0,str.indexOf(".")+1);
			else if (str.length > 50)
				tooltipStr = str.substring(0,47)+"...";
			else
				tooltipStr = str;
		}

		public function showTooltip(... args):void{
			if (tooltip)
				hideTooltip();

			if (toClass(args[0]) == String)
				setTooltipString(args[0]);
			else
				tooltipTimer.stop();

			if (!empty(tooltipStr)){
				var txt:TextField = new TextField();
				txt.text = tooltipStr;
				txt.autoSize = TextFieldAutoSize.LEFT;
		
				var rect:Shape = new Shape;
				rect.graphics.beginFill(0xFFFFFF,1)
				rect.graphics.lineStyle(0.1,0x262626,1);
				rect.graphics.drawRect(0, 0, txt.width,txt.height);
				rect.graphics.endFill();

				tooltip = new MovieClip();
				tooltip.mouseEnabled = false;
				tooltip.mouseChildren = false;
				tooltip.addChild(rect);
				tooltip.addChild(txt);
				tooltip.x = (mouseX+15 + tooltip.width <= 800 ? mouseX+15 : mouseX-5 - tooltip.width);
				tooltip.y = (mouseY+5 + tooltip.height <= 480 ? mouseY+5 : mouseY-5 - tooltip.height);
				new Tween(tooltip,"alpha",None.easeNone,0,1,0.4,true);
				addChild(tooltip);
			}
		}

		public function hideTooltip(e:Event = null):void{
			tooltipTimer.reset();
			if (tooltip && tooltip.alpha > 0){
				if (tooltip.parent)
					removeChild(tooltip);
			}
		}

		private function addEvents(obj:Object):void{
			obj.addEventListener(MouseEvent.ROLL_OVER,eventMouseOver);
			obj.addEventListener(MouseEvent.ROLL_OUT,eventMouseOut);
			obj.addEventListener(MouseEvent.CLICK,eventMouseClick);
		}

		private function clearList(list:Array):void{
			for (var i:int = 0; i < list.length; i++){
				list[i].removeEventListener(MouseEvent.ROLL_OVER,eventMouseOver);
				list[i].removeEventListener(MouseEvent.ROLL_OUT,eventMouseOut);
				list[i].removeEventListener(MouseEvent.CLICK,eventMouseClick);
				if (list[i].parent)
					list[i].parent.removeChild(list[i]);
			}
		}

		public function updateCharList():void{
			clearList(charList);

			charList = Room.getCurrent().getCharList();
			for (var i:int = 0; i < charList.length; i++){
				charList[i].y = room.y + room.height + 5 + 105 * Math.floor(i/2);
				charList[i].x = room.x + room.width - 85 * (i%2+1);
				addEvents(charList[i]);
				addChild(charList[i]);
			}
		}

		public function updatePlayerItems():void{
			clearList(playerItems);

			playerItems = Player.getPlayer().getItemList();
			for (var i:int = 0; i < playerItems.length; i++){
				playerItems[i].y = 130.8 + 16.35 * i;
				addEvents(playerItems[i]);
				player.addChild(playerItems[i]);
			}

			if (playerItems.length * 16.35 + 23.4 > 68.45)
				player.frame.height = playerItems.length * 16.35 + 142.5;

			player.background.height = player.frame.height - 130.8;
		}

		public function updateRoomItems():void{
			clearList(roomItems);

			roomItems = Room.getCurrent().getItemList();
			for (var i:int = 0; i < roomItems.length; i++){
				roomItems[i].y = 11.7 + 16.35 * i;
				addEvents(roomItems[i]);
				room.addChild(roomItems[i]);
			}

			if (roomItems.length * 16.35 + 23.4 > 81.85)
				room.frame.height = roomItems.length * 16.35 + 23.4;
		}

		public function updateNav():void{
			var nav:Nav = Room.getCurrent().getNav();

			for (var i:int = 0; i < navButtons.length; i++)
				if (navButtons[i]){
					if (nav.getRoom(i))
						navButtons[i].gotoAndStop(1);
					else
						navButtons[i].gotoAndStop(2);
				}
		}

		private static function toClass(... args):Class{
			return Class(getDefinitionByName(getQualifiedClassName(args[0])));
		}

		private static function empty(str:String):Boolean{
			return !str || trim(str).length == 0;
		}

		private static function trim(str:String):String{
			return (str ? str.replace(/^\s+|\s+$/g, "") : "");
		}

		public static function setNavSystem(nav:String):void{
			if (nav){
				nav = nav.toLowerCase();
				if (nav != navSystem && (nav == "compass" || nav == "cross"))
					navSystem = nav;
			}
		}

		public static function getNavSystem():String{
			return navSystem;
		}

		public static function getUI():Interface{
			return ui;
		}
	}
}