﻿package script{

	/*
	 * AINE - Accelerated Interactive Narrator Engine
	 * @version 0.4.2 ALPHA
	 * @author Daniel Svejstrup Christensen
	 * @see http://www.drakkashi.com/aine/ for updates
	 * This software is released under the MIT License <http://www.opensource.org/licenses/mit-license.php>
	 * Copyright © 2014 Drakkashi.com
	 */

	public class Room extends Container{

		private static var current:Room,
						   starting:Room,
						   list:Array = new Array();

		private var nav:Nav,
					charList:Array = new Array();

		public function Room(instance:String = null){
			if (instance)
				this.instance = instance;
			else{
				var i:int = 0;
				do{
					this.instance = "room"+i++;
				}
				while(_Object.getObject(this.instance));
			}

			setName(this.instance);
			nav = new Nav(this);
			list.push(this);
		}

		public function hasCharacter(char:Character):Boolean{
			return charList.indexOf(char) >= 0;
		}

		override public function setName(str:String):void{
			super.setName(str);
			if (this == getCurrent() && Interface.getUI())
				Interface.getUI().updateRoomName();
		}

		override public function setImage(str:String):void{
			super.setImage(str);
			if (this == getCurrent() && Interface.getUI())
				Interface.getUI().updateRoomImage();
		}

		public function enterRoom():void{
			var room:Room = _Event.eventLeaveRoom(this);
			 if (room){
				setCurrent(room);
				_Event.eventEnterRoom(room);
			 }
		}

		public function getNav():Nav{
			return nav;
		}

		public static function setCurrent(room:Room):void{
			_Event.clearDestiny();
			if (room && room != current){
				current = room;

				if (Interface.getUI())
					Interface.getUI().updateRoom();
			}
		}
		
		public static function getCurrent():Room{
			return current;
		}
		
		public static function setStartingRoom(room:Room = null):void{
			if (room)
				starting = room;
			else
				starting = list[0];
			setCurrent(starting);
		}
		
		public static function getStartingRoom():Room{
			return starting;
		}

		public static function count():int{
			return list.length;
		}

		public static function getRoom(str:String):Room{
			for (var i:int = 0; i < list.length; i++)
				if (list[i].instance == str)
					return list[i];
			return null;
		}

		override public function setVar(id:String,val:*, ... args):String{
			if (id == "chars"){
				if (toClass(val) == Character || toClass(val) == Array){

					if (toClass(val) == Array){
						var list:Array = val;
						for (var i:int = 0; i < list.length; i++){
							if (list[i] && toClass(list[i]) != Character){
								return 'Invalid entry in list. All entries has to be a Character object.';
								break;
							}
						}
						
						if (i >= list.length)
							for (i = 0; i < list.length; i++)
								if (list[i])
									this.giveCharacter(list[i]);
					}
					else
						this.giveCharacter(val);
				}
				else if (val)
					return 'Room property "chars" has to be a Character object or a List.';
			}
			else if (Nav.pathIndex(id) >= 0){
				if (toClass(val) == Room)
					nav.setPath(id,val,args[0]);
				else
					return 'Navigation path can only point towards a ' + (toClass(val) == Array ? "single " : "") + 'Room object';
			}
			else
				return super.setVar(id,val);
			return null;
		}

		override public function giveItem(... args):void{
			super.giveItem(args);
			if (this == getCurrent() && Interface.getUI())
				Interface.getUI().updateRoomItems();
		}

		override public function takeItem(item:Item):void{
			super.takeItem(item);
			if (this == getCurrent() && Interface.getUI())
				Interface.getUI().updateRoomItems();
		}

		public function giveCharacter(char:Character):void{
			if (charList.indexOf(char) < 0){
				char.setCurrent(this);
				charList.push(char);

				if (this == getCurrent() && Interface.getUI())
					Interface.getUI().updateCharList();
			}
		}

		public function removeCharacter(char:Character):void{
			if (char && hasCharacter(char)){
				charList.splice(charList.indexOf(char),1);

				if (this == getCurrent() && Interface.getUI())
					Interface.getUI().updateCharList();
			}
		}

		public function getCharList():Array{
			return cloneArray(charList);
		}

		override public function getMethods():Array{
			return super.getMethods().concat(new Array("setNav"));
		}

		override public function getArguments():Array{
			return super.getArguments().concat(new Array("NAV_OVER","NAV_OUT","NAV_CLICK"));
		}

		public static function getTags():Array{
			var tags:Array = new Array();

			for (var i:int = 0; i < list.length; i++)
				tags.push(list[i].getInstance());
			return tags;
		}
	}
}