﻿package script{

	/*
	 * AINE - Accelerated Interactive Narrator Engine
	 * @version 0.4.2 ALPHA
	 * @author Daniel Svejstrup Christensen
	 * @see http://www.drakkashi.com/aine/ for updates
	 * This software is released under the MIT License <http://www.opensource.org/licenses/mit-license.php>
	 * Copyright © 2014 Drakkashi.com
	 */

	import flash.display.MovieClip;
	import flash.utils.getQualifiedClassName;
	import flash.utils.getDefinitionByName;
	import flash.events.Event;

	public class _Object extends MovieClip{

		protected var instance:String,
					  displayName:String,
					  image:String,
					  desc:String,
					  tooltip:String,
					  varList:Array = new Array(new Array(), new Array()),
					  eventList:Array = new Array(),
					  customList:Array = new Array();
	
		protected function getInstance():String{
			return instance;
		}

		public function setName(str:String):void{
			if(!empty(str))
				displayName = str;
		}

		public function getName():String{
			return displayName;
		}

		public function setImage(str:String):void{
			if(!empty(str) && Image.getImage(str))
				image = str;
		}

		public function getImage():String{
			return image;
		}

		public function setDesc(str:String):void{
			if(!empty(str))
				desc = str;
		}

		public function getDesc():String{
			return desc;
		}
	
		public function setTooltip(str:String):void{
			if(!empty(str))
				tooltip = str;
		}

		public function getTooltip():String{
			if (!empty(tooltip))
				return tooltip;
			if (!empty(getDesc()))
				return getDesc();
			return getName();
		}

		public function setVar(id:String,val:*, ... args):String{
			if (toClass(val) == Array){
				var list:Array = val,
					str:String = "";

				for(var i:int = 0; i < list.length; i++)
					str += String(list[i]);
				val = str;
			}

			if (id == "name")
				setName(String(val));
			else if (id == "image")
				setImage(String(val));
			else if (id == "desc")
				setDesc(String(val));
			else if (id == "tooltip")
				setTooltip(String(val));
			else{
				var index:int = varList[0].indexOf(trim(id));

				if (index < 0){
					varList[0].push(id);
					varList[1].push(val);
				}
				else
					varList[1][index] = val;
			}
			return null;
		}

		public function getVar(str:String):String{
			var index:int = varList[0].indexOf(str);
			if (index < 0)
				return null;
			else
				return varList[1][index];
		}

		override public function toString():String{
			return getInstance();
		}

		public function triggerEvent(str:String):Boolean{
			var evt:_Event = getEvent(str);

			if (evt){
				if (evt.isEnabled())
					evt.trigger();
				return evt.getOverride();
			}
			return false;
		}

		public function getCustomList():Array{
			var list:Array = new Array();

			for (var i:int = 0; i < customList.length; i++)
				if (customList[i].isEnabled())
					list.push(customList[i]);
					
			return list;
		}

		public function getEvent(str:String):_Event{
			var list:Array = cloneArray(eventList).concat(customList);
			for (var i:int = 0; i < list.length; i++)
				if (list[i] && list[i].getEvent() == str)
					return list[i];
			return null;
		}

		public function enableEvent(str:String):void{
			var evt:_Event = getEvent(str);

			if (evt)
				evt.enable();
			else
				Output.print("ERROR: Unable to enable undefined Event");
		}

		public function disableEvent(str:String):void{
			var evt:_Event = getEvent(str);
			
			if (evt)
				evt.disable();
			else
				Output.print("ERROR: Unable to disable undefined Event");
		}

		public function addEvent(newEvt:_Event,b:Boolean):void{
			var evt:_Event = getEvent(newEvt.getEvent());
			
			if (evt && newEvt.getOverride()){
				if (eventList.indexOf(evt) >= 0)
					eventList[eventList.indexOf(evt)] = newEvt;
				else
					customList[customList.indexOf(evt)] = newEvt;
			}
			else if (evt){
				evt.append(newEvt.getString());
				if (b && evt.isEnabled() != newEvt.isEnabled())
					evt.toggle();
			}
			else{
				var index:int = getArguments().indexOf(newEvt.getEvent());
				if (index < 0)
					customList.push(newEvt);
				else
					eventList[index] = newEvt;
			}
		}

		public function getArguments():Array{
			return new Array(
						"ENTER_ROOM_FIRST","LEAVE_ROOM_FIRST","ENTER_ROOM","LEAVE_ROOM",
						"MOUSE_OVER","MOUSE_OUT","MOUSE_CLICK"
					);
		}

		public function getMethods():Array{
			return new Array("setName","getName","setImage","getImage","setDesc","getDesc","setTooltip","getTooltip","setVar","getVar","enableEvent","disableEvent");
		}

		public function getClass():Class{
			return Class(getDefinitionByName(getQualifiedClassName(this)));
		}

		protected static function toClass(instance:*):Class{
			if (!instance)
				return null;
			return Class(getDefinitionByName(getQualifiedClassName(instance)));
		}

		protected static function empty(str:String):Boolean{
			return !str || trim(str).length == 0;
		}

		protected static function trim(str:String):String{
			return (str ? str.replace(/^\s+|\s+$/g, "") : "");
		}

		protected static function cloneArray(array:Object):Array {
			return array.concat();
		}

		public static function getObject(str:String):_Object{
			if (!str)
				return null;

			var obj:_Object = Player.getPlayer(str);

			if (!obj)
				obj = Item.getItem(str);
			
			if (!obj)
				obj = Room.getRoom(str);

			if (!obj)
				obj = Character.getCharacter(str);

			return obj;
		}
	}
}